#include "common.h"
#include "LabelSettings.h"
#include "Font.h"
#include "Texture.h"

NameValuePair alignValues[] = {
	{ "left", DT_LEFT },
	{ "center", DT_CENTER },
	{ "right", DT_RIGHT },
	{ 0, 0 }
};

NameValuePair vertAlignValues[] = {
	{ "top", ALIGN_TOP },
	{ "center", ALIGN_CENTER },
	{ "bottom", ALIGN_BOTTOM },
	{ 0, 0 }
};

NameValuePair keyValues[] = {
	{ "ctrl", 1 },
	{ "shift", 2 },
	{ "ctrl+shift", 3 },
	{ "shift+ctrl", 3 },
	{ 0, 0 }
};

NameValuePair buttonValues[] = {
	{ "left", 1 },
	{ "right", 2 },
	{ "middle", 3 },
	{ 0, 0 }
};

NameValuePair autoWidthValues[] = {
	{ "left", 1 },
	{ "right", 2 },
	{ "center", 3 },
	{ 0, 0 }
};

NameValuePair autoHeightValues[] = {
	{ "top", 1 },
	{ "bottom", 2 },
	{ "center", 3 },
	{ 0, 0 }
};

NameValuePair transparencyValues[] = {
	{ "true", 1 },
	{ "fake", 2 },
	{ "false", 3 },
	{ "off", 3 },
	{ "none", 3 },
	{ ".none", 3 },
	{ 0, 0 }
};

NameValuePair effectValues[] = {
	{ "Random", -1},
	{ "Spin", 0},
	{ "Vortex", 1},
	{ "ScatterGather", 2},
	{ "Spike", 3},
	{ "Fireworks", 4},
	{ 0, 0 }
};

LabelSettings::LabelSettings()
{
	const char *name = "AllLabels";
	isDefault = true;

	alwaysOnTop = GetRCBoolean(name, "AlwaysOnTop");
	startHidden = GetRCBoolean(name, "StartHidden");
	bUseFahrenheit = GetRCBoolean(name, "UseFahrenheit");

	skin = GetRCTexture(name, "");

	font = GetRCFont(name);
	
	leftBorder = GetRCInt(name, "LeftBorder", 0);
	topBorder = GetRCInt(name, "TopBorder", 0);
	rightBorder = GetRCInt(name, "RightBorder", 0);
	bottomBorder = GetRCInt(name, "BottomBorder", 0);

	align = GetRCNamedValue(name, "Align", alignValues, DT_CENTER);
	vertAlign = GetRCNamedValue(name, "VertAlign", vertAlignValues, ALIGN_CENTER);
	updateInterval = GetRCInt(name, "UpdateInterval", 1000);
	alwaysUpdateContent = GetRCBoolean(name, "UpdateAlways");

	int screenX = GetSystemMetrics(SM_CXSCREEN);
	int screenY = GetSystemMetrics(SM_CYSCREEN);

	width = GetRCDimen(name, "Width", 64, screenX);
	height = GetRCDimen(name, "Height", 64, screenY);
	x = GetRCCoord(name, "X", 0, screenX);
	y = GetRCCoord(name, "Y", 0, screenY);

	text = GetRCString(name, "Text", "");
	tooltip = GetRCString(name, "Tooltip", "");

	lineBreak = GetRCBoolean(name, "AutoLineBreak");

	leftClickCommand = GetRCLine(name, "OnLeftClick", "");
	leftDoubleClickCommand = GetRCLine(name, "OnLeftDoubleClick", "");
	middleClickCommand = GetRCLine(name, "OnMiddleClick", "");
	middleDoubleClickCommand = GetRCLine(name, "OnMiddleDoubleClick", "");
	rightClickCommand = GetRCLine(name, "OnRightClick", "");
	rightDoubleClickCommand = GetRCLine(name, "OnRightDoubleClick", "");
	wheelDownCommand = GetRCLine(name, "OnWheelDown", "");
	wheelUpCommand = GetRCLine(name, "OnWheelUp", "");
	enterCommand = GetRCLine(name, "OnEnter", "");
	leaveCommand = GetRCLine(name, "OnLeave", "");
	dropCommand = GetRCLine(name, "OnDrop", "");
	textChangeCommand = GetRCLine(name, "OnTextChange", "");

	scrollPadLength = GetRCInt(name, "scrollPad", 10); 
	scrollInterval = GetRCInt(name, "scrollInterval", 100);
	scrollSpeed = GetRCInt(name, "scrollSpeed", 1);
	scroll = GetRCBoolean(name, "scroll");

	autoWidthMode = GetRCNamedValue(name, "AutoWidthMode", autoWidthValues, 0);
	autoHeightMode = GetRCNamedValue(name, "AutoHeightMode", autoHeightValues, 0);
	autoMaxWidth = GetRCDimen(name, "AutoMaxWidth", screenX, screenX);
	autoMaxHeight = GetRCDimen(name, "AutoMaxHeight", screenY, screenY);

	transparencyMode = GetRCNamedValue(name, "TransparencyMode", transparencyValues, 1);
	alphaTransparency = GetRCInt(name, "AlphaTransparency", 255, 0, 255);
	alphaFade = GetRCBoolean(name, "AlphaFade");
	ghosted = GetRCBoolean(name, "Ghosted");

	moveable = GetRCBoolean(name, "Moveable");
	moveKey = GetRCNamedValue(name, "MoveModifierKey", keyValues, 1);
	moveButton = GetRCNamedValue(name, "MoveButton", buttonValues, 1);

	string helper;
	helper = "*";
	helper.append(name);
	helper.append("OnLeftClickRegion");
	labelLeftClickRegions = GetRCLineList(helper.c_str());
	helper = "*";
	helper.append(name);
	helper.append("OnRightClickRegion");
	labelRightClickRegions = GetRCLineList(helper.c_str());
	helper = "*";
	helper.append(name);
	helper.append("OnMiddleClickRegion");
	labelMiddleClickRegions = GetRCLineList(helper.c_str());

	enterleaveRegion = GetRCLine(name, "OnEnterLeaveRegion", "0 0 -0 -0");

	labelGroup = GetRCString(name, "AddToGroup", "");

	htmltext = GetRCBoolean(name, "PluginHtmlText");
	anim = GetRCBoolean(name, "PluginAnim");
	animEffect = GetRCNamedValue(name, "AnimEffect", effectValues, 1);
	animSteps = GetRCInt(name, "AnimSteps", 50, 1, 255);
	animDelay = GetRCInt(name, "AnimDelay", 5, 0, 100);
	animAfterSteps = GetRCInt(name, "AnimAfterSteps", 6, 0, 32);
}

LabelSettings::LabelSettings(const char *name)
{
	isDefault = false;

	alwaysOnTop = GetRCBoolean(name, "AlwaysOnTop", defaultSettings->alwaysOnTop);
	startHidden = GetRCBoolean(name, "StartHidden", defaultSettings->startHidden);
	bUseFahrenheit = GetRCBoolean(name, "UseFahrenheit", defaultSettings->bUseFahrenheit);

	skin = GetRCTexture(name, "", defaultSettings->skin);

	font = GetRCFont(name, defaultSettings->font);
	
	leftBorder = GetRCInt(name, "LeftBorder", defaultSettings->leftBorder);
	topBorder = GetRCInt(name, "TopBorder", defaultSettings->topBorder);
	rightBorder = GetRCInt(name, "RightBorder", defaultSettings->rightBorder);
	bottomBorder = GetRCInt(name, "BottomBorder", defaultSettings->bottomBorder);

	align = GetRCNamedValue(name, "Align", alignValues, defaultSettings->align);
	vertAlign = GetRCNamedValue(name, "VertAlign", vertAlignValues, defaultSettings->vertAlign);
	updateInterval = GetRCInt(name, "UpdateInterval", defaultSettings->updateInterval);
	alwaysUpdateContent = GetRCBoolean(name, "UpdateAlways", defaultSettings->alwaysUpdateContent);

	int screenX = GetSystemMetrics(SM_CXSCREEN);
	int screenY = GetSystemMetrics(SM_CYSCREEN);

	width = GetRCDimen(name, "Width", defaultSettings->width, screenX);
	height = GetRCDimen(name, "Height", defaultSettings->height, screenY);
	x = GetRCCoord(name, "X", defaultSettings->x, screenX);
	y = GetRCCoord(name, "Y", defaultSettings->y, screenY);

	text = GetRCString(name, "Text", defaultSettings->text);
	tooltip = GetRCString(name, "Tooltip", defaultSettings->tooltip);

	lineBreak = GetRCBoolean(name, "AutoLineBreak", defaultSettings->lineBreak);

	leftClickCommand = GetRCLine(name, "OnLeftClick", defaultSettings->leftClickCommand);
	leftDoubleClickCommand = GetRCLine(name, "OnLeftDoubleClick", defaultSettings->leftDoubleClickCommand);
	middleClickCommand = GetRCLine(name, "OnMiddleClick", defaultSettings->middleClickCommand);
	middleDoubleClickCommand = GetRCLine(name, "OnMiddleDoubleClick", defaultSettings->middleDoubleClickCommand);
	rightClickCommand = GetRCLine(name, "OnRightClick", defaultSettings->rightClickCommand);
	rightDoubleClickCommand = GetRCLine(name, "OnRightDoubleClick", defaultSettings->rightDoubleClickCommand);
	wheelDownCommand = GetRCLine(name, "OnWheelDown", defaultSettings->wheelDownCommand);
	wheelUpCommand = GetRCLine(name, "OnWheelUp", defaultSettings->wheelUpCommand);
	enterCommand = GetRCLine(name, "OnEnter", defaultSettings->enterCommand);
	leaveCommand = GetRCLine(name, "OnLeave", defaultSettings->leaveCommand);
	dropCommand = GetRCLine(name, "OnDrop", defaultSettings->dropCommand);
	textChangeCommand = GetRCLine(name, "OnTextChange", defaultSettings->textChangeCommand);

	scrollPadLength = GetRCInt(name, "scrollPad", defaultSettings->scrollPadLength);
	scrollInterval = GetRCInt(name, "scrollInterval", defaultSettings->scrollInterval);
	scrollSpeed = GetRCInt(name, "scrollSpeed", defaultSettings->scrollSpeed);
	scroll = GetRCBoolean(name, "scroll", defaultSettings->scroll);

	autoWidthMode = GetRCNamedValue(name, "AutoWidthMode", autoWidthValues, defaultSettings->autoWidthMode);
	autoHeightMode = GetRCNamedValue(name, "AutoHeightMode", autoHeightValues, defaultSettings->autoHeightMode);
	autoMaxWidth = GetRCDimen(name, "AutoMaxWidth", defaultSettings->autoMaxWidth, screenX);
	autoMaxHeight = GetRCDimen(name, "AutoMaxHeight", defaultSettings->autoMaxHeight, screenY);

	transparencyMode = GetRCNamedValue(name, "TransparencyMode", transparencyValues, defaultSettings->transparencyMode);
	alphaTransparency = GetRCInt(name, "AlphaTransparency", defaultSettings->alphaTransparency);
	alphaFade = GetRCBoolean(name, "AlphaFade", defaultSettings->alphaFade);
	ghosted = GetRCBoolean(name, "Ghosted", defaultSettings->ghosted);

	moveable = GetRCBoolean(name, "Moveable", defaultSettings->moveable);
	moveKey = GetRCNamedValue(name, "MoveModifierKey", keyValues, defaultSettings->moveKey);
	moveButton = GetRCNamedValue(name, "MoveButton", buttonValues, defaultSettings->moveButton);

	string helper;
	helper = "*";
	helper.append(name);
	helper.append("OnLeftClickRegion");
	labelLeftClickRegions = GetRCLineList(helper.c_str());
	helper = "*";
	helper.append(name);
	helper.append("OnRightClickRegion");
	labelRightClickRegions = GetRCLineList(helper.c_str());
	helper = "*";
	helper.append(name);
	helper.append("OnMiddleClickRegion");
	labelMiddleClickRegions = GetRCLineList(helper.c_str());

	enterleaveRegion = GetRCLine(name, "OnEnterLeaveRegion", defaultSettings->enterleaveRegion);

	labelGroup = GetRCString(name, "AddToGroup", defaultSettings->labelGroup);

	htmltext = GetRCBoolean(name, "PluginHtmlText", defaultSettings->htmltext);
	anim = GetRCBoolean(name, "PluginAnim", defaultSettings->anim);
	animEffect = GetRCNamedValue(name, "AnimEffect", effectValues, defaultSettings->animEffect);
	animSteps = GetRCInt(name, "AnimSteps", defaultSettings->animSteps, 1, 255);
	animDelay = GetRCInt(name, "AnimDelay", defaultSettings->animDelay, 0, 100);
	animAfterSteps = GetRCInt(name, "AnimAfterSteps", defaultSettings->animAfterSteps, 0, 32);
}

LabelSettings::~LabelSettings()
{
	if(isDefault)
	{
		delete skin;
		delete font;
	}
}

LabelSettings* defaultSettings = NULL;
